<?php

use v2\Shop\Shop;
use v2\Models\Wallet;
use v2\Models\Document;
use v2\Models\Commission;
use v2\Models\Investment;
use v2\Models\Withdrawal;
use v2\Security\TwoFactor;
use v2\Models\PayoutWallet;
use v2\Classes\ExchangeRate;
use Filters\Filters\OrderFilter;
use v2\Models\InvestmentPackage;
use Filters\Filters\WalletFilter;
use Filters\Filters\WithdrawalFilter;
use Filters\Filters\SupportTicketFilter;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Database\Capsule\Manager as DB;

/**
 *
 */
class UserController extends controller
{

    public function __construct()
    {
        if (!$this->admin()) {
            $this->middleware('current_user')
                ->mustbe_loggedin()
                // ->must_have_verified_email()
            ;
            // ->must_have_verified_company();
        }
    }


    public function choose_membership($membership_id)
    {
        $auth = $this->auth();
        $membership = SubscriptionPlan::find($membership_id);

        if ($membership == null) {
            return;
        }

        $subscription_id = $membership_id;


        $personal_settings = $auth->SettingsArray;
        $personal_settings['membership_choice'] = $membership_id;

        $auth->save_settings($personal_settings);

        if ($membership_id < 2) {

            Redirect::back();
        }

        $response = SubscriptionPlan::create_subscription_request($subscription_id, $auth->id,  true);

        Redirect::back();
    }


    public function membership_orders()
    {
        $this->view('auth/membership_orders');
    }


    public function direct_ranks()
    {
        $direct_ranks = $this->auth()->referred_members_downlines(1)[1];
        $direct_ranks = User::whereIn('id', collect($direct_ranks)->where('rank', '>', -1)->pluck('id')->toArray())->get();
        $this->view('auth/direct_ranks', compact('direct_ranks'));
    }

    public function send_email_code()
    {
        echo "<pre>";

        $this->create_email_code();
    }


    public function resources($category_key = null)
    {

        $category = Document::$categories[$category_key] ?? null;

        $documents = Document::where('category', $category)->get();
        $title = "$category";

        if ($documents->isEmpty()) {
            $documents = Document::get();
            $title = "All Documents";
        }

        $this->view('auth/resources', compact('title', 'documents'));
    }

    public function faqs()
    {
        $this->view('auth/faqs');
    }

    public function supportmessages($value = '')
    {
        $this->view('auth/support-messages');
    }




    public function submit_2fa()
    {
        $auth = $this->auth();

        if ($_POST['code'] == '') {
            Session::putFlash('danger', "Invalid Code");
            Redirect::back();
        }

        $this->verify_2fa_only();


        $existing_settings = $auth->SettingsArray;

        $twofa_recovery = MIS::random_string(10);
        if (!$auth->has_2fa_enabled()) {
            $existing_settings['enable_2fa'] = 1;
            $existing_settings['2fa_recovery'] = $twofa_recovery;

            Session::putFlash('success', "2FA enabled successfully");
        } else {
            $existing_settings['enable_2fa'] = 0;
            Session::putFlash('success', "2FA disabled successfully");
        }

        $auth->save_settings($existing_settings);

        Redirect::back();
    }

    public function two_factor_authentication()
    {

        $auth = $this->auth();

        if ($auth->has_2fa_enabled()) {

            $image = null;
        } else {

            $_2FA = new TwoFactor($auth);
            $image = $_2FA->getQrCode();
        }

        $this->view('auth/two-factor-authentication', compact('image'));
    }



    public function submit_hash_rate($order_id = '')
    {
        echo "<Pre>";
        print_r($_POST);
        $auth = $this->auth();
        $order = Wallet::where('user_id', $auth->id)->where('id', $_POST['order_id'])->first();


        if ($order == null) {
            Session::putFlash("danger", "Invalid Request");
            Redirect::back();
        }

        $extra_detail = $order->ExtraDetailArray;
        $extra_detail['hash_rate'] = $_POST['hash_rate'];


        $order->update([
            'extra_detail' => json_encode($extra_detail)
        ]);

        Redirect::back();
    }



    public function submit_make_deposit()
    {


        $rules_settings = SiteSettings::find_criteria('rules_settings');
        $min_deposit = $rules_settings->settingsArray['min_deposit_usd'];

        $this->validator()->check(Input::all(), array(
            'amount' => [
                'required' => true,
                'positive' => true,
                'min_value' => $min_deposit,
            ],
            'payment_method' => [
                'required' => true,
            ],
        ));


        if (!$this->validator->passed()) {

            Session::putFlash('danger', Input::inputErrors());
            Redirect::back();
        }

        DB::beginTransaction();

        try {


            $deposit = Wallet::create([
                'user_id' => $this->auth()->id,
                'amount' => Input::get('amount'),
                'earning_category' => 'deposit',
                'comment' => 'Funding Account',
                'type' => 'credit',
                'status' => 'pending',
                'payment_method' => $_POST['payment_method'],
            ]);

            DB::commit();
        } catch (Exception $e) {
            DB::rollback();
            Session::putFlash("danger", "We could not initialize the payment process. Please try again");
            Redirect::back();
        }

        $callback_param = http_build_query([
            'item_purchased' => $deposit->name_in_shop,
            'order_unique_id' => $deposit->id,
            'payment_method' => $deposit->payment_method,
        ]);


        $callback_url = "shop/checkout?$callback_param";


        /*		$deposit->mark_paid();
                $shop = new Shop();
                $shop->empty_cart_in_session();
        */

        Redirect::to("$callback_url");


        // $this->deposit_checkout($deposit->id);

    }



    public function notifications($notification_id = 'all')
    {

        $auth = $this->auth();
        $per_page = 50;
        $page = $_GET['page'] ?? 1;

        switch ($notification_id) {
            case 'all':
                $notifications = Notifications::all_notifications($auth->id, $per_page, $page);
                $total = Notifications::all_notifications($auth->id)->count();
                break;

            default:

                $total = null;

                $notifications = Notifications::where('user_id', $auth->id)->where('id', $notification_id)->first();

                Notifications::mark_as_seen([$notifications->id]);


                if ($notifications == null) {
                    Session::putFlash("danger", "Invalid Request");
                    Redirect::back();
                }



                if ($notifications->DefaultUrl != $notifications->UsefulUrl) {

                    Redirect::to($notifications->UsefulUrl);
                }

                break;
        }



        $this->view('auth/notifications', compact('notifications', 'per_page', 'total'));
    }



    public function commission_history()
    {

        $auth = $this->auth();

        $sieve = $_REQUEST;
        $sieve = array_merge($sieve);

        $query = Commission::for($auth->id)->latest();
        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $filter = new  WalletFilter($sieve);

        $balance = $query->sum('amount');

        $total = $query->count();

        $data = $query->Filter($filter)->count();

        $sql = $query->Filter($filter);

        $records = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered


        $balance = Commission::availableBalanceOnUser($auth->id);

        $note = MIS::filter_note($records->count(), $data, $total,  $sieve, 1);

        $this->view('auth/commission_history', compact('records', 'balance', 'sieve', 'data', 'per_page', 'note'));
    }




    public function company()
    {
        $company = $this->auth()->company;
        $this->view('auth/company', compact('company'));
    }


    public function order($order_id = null)
    {

        $order = SubscriptionOrder::where('id', $order_id)->where('user_id', $this->auth()->id)->first();
        echo $this->buildView('auth/order_detail', compact('order'));
    }



    public function products()
    {
        $this->view('auth/products');
    }


    public function products_orders()
    {
        $this->view('auth/products_orders');
    }


    public function view_cart()
    {

        $cart = json_decode($_SESSION['cart'], true)['$items'];

        if (count($cart) == 0) {
            Session::putFlash("info", "Your cart is empty.");
            Redirect::to('user/shop');
        }
        $this->view('auth/view_cart');
    }


    public function shop()
    {

        $products = $this->auth()->accessible_products();

        $this->view('auth/shop', compact('products'));
    }


    public function scheme()
    {

        $subscription = $this->auth()->subscription;

        if ($subscription == null) {

            Redirect::back();
        }

        $this->view('auth/subscription_confirmation', compact('subscription'));
    }



    public function create_upgrade_request($subscription_id = null)
    {

        $subscription_id = $_REQUEST['subscription_id'];

        $response = SubscriptionPlan::create_subscription_request($subscription_id, $this->auth()->id);


        header("content-type:application/json");
        echo $response;

        Redirect::back();
    }


    public function deposit_orders_history()
    {
        $deposit_orders = Wallet::Category('deposit')->where('user_id', $this->auth()->id)->Credit()->Pending()->Unpaid()->latest()->get();

        $this->view('auth/deposit_orders_history', compact('deposit_orders'));
    }


    public function invoices()
    {
        $packs = InvestmentPackage::for($this->auth()->id)->latest()->get();

        $this->view('auth/invoices', compact('packs'));
    }



    public function transfer_history()
    {

        $rules_settings = SiteSettings::find_criteria('rules_settings');
        $setting = $rules_settings->settingsArray;
        $trucash_exchange = $setting['one_trucash_is_x_usd'];


        $auth = $this->auth();

        $sieve = $_REQUEST;
        $sieve = array_merge($sieve);

        $query = Wallet::for($auth->id)->latest()->where('payment_method', 'transfer');

        $total_transfer_from_deposit = $query->count();
        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $filter = new  WalletFilter($sieve);

        $data = $query->Filter($filter)->count();

        $sql = $query->Filter($filter);


        $records = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered



        $query = PayoutWallet::for($auth->id)->latest();


        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve, ['comment' => 'Transfer']);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $filter = new  WalletFilter($sieve);

        $p_data = $query->Filter($filter)->count();
        $total_transfer_from_payout = $query->Filter($filter)->count();

        $sql = $query->Filter($filter);

        $from_payout_wallet = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered



        $records = $records->concat($from_payout_wallet)->sortByDesc('created_at');



        $total = $total_transfer_from_payout + $total_transfer_from_deposit;
        $data = $data + $p_data;


        $note = MIS::filter_note($records->count(), $data, $total,  $sieve, 1);


        $this->view('auth/transfer_history', compact('records', 'note', 'trucash_exchange', 'sieve', 'data', 'per_page'));
    }


    public function investments()
    {
        $auth = $this->auth();

        $investments = Investment::where('user_id', $auth->id)->latest()->get();

        $this->view('auth/investments', compact('investments'));
    }


    public function select_pack()
    {

        $investment = InvestmentPackage::find($_POST['investment_id']);

        if ($investment == null) {
            Session::putFlash('danger', "Invalid Request");
            Redirect::back();
        }

        $wallet = new Wallet;

        $this->view('auth/select_pack', compact('investment', 'wallet'));
    }

    public function submit_investment()
    {

        echo "<pre>";
        $pack = InvestmentPackage::find($_POST['pack_id']);



        if ($pack == null) {
            Session::putFlash('danger', "Invalid Request");
            echo "not in range";
            // Redirect::back();
        }


        $this->validator()->check(Input::all(), array(

            'wallet' => [
                // 'required' => true,
            ],
            'pack_id' => [
                'required' => true,
            ],
        ));



        if (!$this->validator->passed()) {
            Session::putFlash('danger', Input::inputErrors());
            Redirect::back();
        }

        $auth = $this->auth();

        $amount = $pack->DetailsArray['min_capital'];

        DB::beginTransaction();


        try {
            //debit this user

            //debit user
            $comment = "Purchased $pack->name for $amount";
            $debit = Wallet::createTransaction(
                'debit',
                $auth->id,
                null,
                $amount,
                'completed',
                'deposit',
                $comment,
                null,
                null,
                null
            );

            if ($debit == false) {

                throw new Exception("Could not debit", 1);
            }





            //create investment

            $investment = Investment::create([
                'user_id' => $auth->id,
                'pack_id' => $pack->id,
                'capital' => $amount,
                'worth_after_maturity' => $pack->getWorthAfterMaturity()['roi_and_capital'],
                'currency_id' => null,
                'matures_at' => $pack->getMaturityTimeFrom(),
                'status' => 1,
                'extra_detail' => $pack->toJson(),
            ]);


            if ($investment == false) {
                throw new Exception("Could not create investment", 1);
            }


            $debit->update([
                'payment_method' => 'pack',
                'order_id' =>  $investment->id,
            ]);


            $investment->give_referral_commission();

            DB::commit();
            Session::putFlash('success', "{$investment->pack->name} purchased successfully");
        } catch (Exception $e) {
            DB::rollback();
            // print_r($e->getMessage());
            Session::putFlash('danger', 'Action Failed');
        }


        Redirect::to("user/purchase-investment");
    }


    public function purchase_investment()
    {
        $shop = new Shop;
        $wallet = new Wallet;
        $this->view('auth/purchase_investment', compact('shop', 'wallet'));
    }

    public function account_plan()
    {
        $wallet = new Wallet;
        $this->view('auth/account_plan', compact('wallet'));
    }


    public function reports()
    {
        $this->view('auth/report');
    }


    public function submit_user_transfers()
    {
        echo "<pre>";



        $rules_settings = SiteSettings::find_criteria('rules_settings');
        $transfer_fee = $rules_settings->settingsArray['user_transfer_fee_percent'];
        $min_transfer = $rules_settings->settingsArray['min_transfer_usd'];


        $this->validator()->check(Input::all(), array(
            'amount' => [
                'required' => true,
                'positive' => true,
                'min_value' => $min_transfer,
            ],
            'wallet' => [
                'required' => true,
            ],

            'username' => [
                'required' => true,
                'exist' => 'User|username',
            ],

        ));


        if (!$this->validator->passed()) {

            Session::putFlash('danger', Input::inputErrors());
            Redirect::back();
        }

        $auth = $this->auth();
        $from = $auth->id;
        $amount = Input::get('amount');
        $username = Input::get('username');
        $wallet = Input::get('wallet');

        $to = User::where('username', $username)->first()->id;


        $wallet_to_use = Wallet::$wallets[$_POST['wallet']];
        $wallet_class = $wallet_to_use['class'];
        $wallet_category = $wallet_to_use['category'];


        $transfer = $wallet_class::makeTransfer($from, $to, $amount, $wallet_category, 'deposit');
        $currency = Config::currency();
        $formatted_amount = MIS::money_format($amount);

        if ($transfer == true) {

            Session::putFlash('success', "$currency$formatted_amount transfer initiated successfully to $username");
        } else {

            Session::putFlash('danger', "Transfer Failed");
        }

        Redirect::back();
    }








    public function user_transfers()
    {
        $auth = $this->auth();
        $wallet = new Wallet;
        $deposit_balance = Wallet::availableBalanceOnUser($auth->id, 'deposit');

        $this->view('auth/user-transfers', compact('wallet', 'deposit_balance'));
    }


    public function make_deposit()
    {
        $auth = $this->auth();
        $shop = new Shop;
        $deposits = Wallet::for($auth->id)->Category('deposit')->Credit()->latest()->get();
        $deposit_balance = Wallet::TraitavailableBalanceOnUser($auth->id, 'deposit');

        $this->view('auth/make_deposit', compact('shop', 'deposit_balance', 'deposits'));
    }


    public function my_wallet()
    {
        $this->view('auth/my_wallet');
    }


    public function make_withdrawal()
    {
        $this->view('auth/make_withdrawal');
    }


    public function withdrawals()
    {

        $query = Withdrawal::where('user_id', $this->auth()->id)->latest();


        $sieve = $_REQUEST;
        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $filter = new  WithdrawalFilter($sieve);

        $data = $query->Filter($filter)->count();

        $withdrawals = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered




        $this->view('auth/withdrawal-history', compact('withdrawals', 'sieve', 'data', 'per_page'));



        // $this->view('auth/withdrawal-history', compact('withdrawals'));
    }



    public function fetch_testimonial($testimony_id)
    {
        $testimony = Testimonials::find($testimony_id);
        header("content-type:application/json");

        echo $testimony;
    }

    public function update_testimonial()
    {

        echo "<pre>";
        $testimony_id = Input::get('testimony_id');

        $auth = $this->auth();

        $testimony = Testimonials::where('user_id', $auth->id)->where('id', $testimony_id)->NotApproved()->first();

        $attester = $auth->lastname . ' ' . $auth->firstname;

        Testimonials::updateOrCreate(
            [
                'id' => $_POST['testimony_id']
            ],
            [
                'attester' => $attester,
                'user_id' => $auth->id,
                'content' => Input::get('testimony'),
                'type' => Input::get('type'),
                'video_link' => Input::get('video_link'),
                'intro' => Input::get('intro'),
                'approval_status' => 0
            ]
        );


        Session::putFlash('success', 'Testimonial updated successfully. Awaiting approval');

        Redirect::back();
    }


    public function create_testimonial()
    {
        if (Input::exists() || true) {

            $auth = $this->auth();

            $testimony = Testimonials::create([
                'attester' => $auth->lastname . ' ' . $auth->firstname,
                'user_id' => $auth->id,
                'content' => Input::get('testimony')
            ]);
        }
        Redirect::to("user/edit_testimony/{$testimony->id}");
    }


    public function edit_testimony($testimony_id = null)
    {
        $testimony = Testimonials::where('user_id', $this->auth()->id)->where('id', $testimony_id)->NotApproved()->first();

        if (($testimony == null)) {
            Session::putFlash('danger', 'Invalid Request');
            Redirect::back();
        }


        $this->view('auth/edit_testimony', ['testimony' => $testimony]);
    }


    public function view_testimony()
    {
        $this->view('auth/view-testimony');
    }


    public function testimony()
    {

        $auth = $this->auth();
        $testimonials = Testimonials::where('user_id', $auth->id)->latest()->get();
        $this->view('auth/testimony', compact('testimonials'));
    }


    public function documents()
    {
        $show = false;
        $this->view('auth/documents', compact('show'));
    }


    public function news()
    {
        $this->view('auth/news');
    }

    public function language()
    {
        $this->view('auth/language');
    }


    public function profile()
    {
        $this->view('auth/profile');
    }

    public function upload_payment_proof()
    {
        $order_id = $_POST['order_id'];
        $order = SubscriptionOrder::find($order_id);
        $order->upload_payment_proof($_FILES['payment_proof']);
        Session::putFlash('success', "#$order_id Proof Uploaded Successfully!");
        Redirect::back();
    }

    public function contact_us()
    {
        $this->view('auth/contact-us');
    }


    public function support()
    {
        $auth = $this->auth();

        $sieve = $_REQUEST;
        $sieve = array_merge($sieve);

        $query = SupportTicket::where('user_id', $auth->id)->latest();
        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $filter = new  SupportTicketFilter($sieve);

        $data = $query->Filter($filter)->count();

        $tickets = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered


        $this->view('auth/support', compact('tickets', 'sieve', 'data', 'per_page'));
    }


    public function view_ticket($ticket_id)
    {

        $support_ticket = SupportTicket::find($ticket_id);

        $this->view('auth/support-messages', [
            'support_ticket' => $support_ticket
        ]);
    }


    public function index()
    {
        $this->dashboard();
    }


    public function accounts()
    {
        $this->view('auth/accounts');
    }


    public function change_password()
    {
        $this->accounts();
    }


    public function games()
    {
        $auth = $this->auth();

        $query = Orders::where('user_id', $auth->id)/*->whereIn('id', [2])*/->Paid()->latest();

        $sieve = $_REQUEST;
        // $sieve = array_merge($sieve, $extra_sieve);
        $total = $query->count();
        // ->where('status', 1);  //in review
        $sieve = array_merge($sieve);
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 20;
        $skip = (($page - 1) * $per_page);

        $filter = new OrderFilter($sieve);

        $data = $query->Filter($filter)->count();

        $orders = $query->Filter($filter)
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered

        $note = MIS::filter_note($orders->count(), $data, $total,  $sieve, 1);


        foreach ($orders as $key => $order) {
            //this makes live edit of ad reflect in order purchases
            $order->order_detail = $order->delivery_details();
        }

        $shop = new Shop;

        $this->view('auth/games', compact('orders', 'note', 'total', 'per_page', 'sieve', 'shop'));
    }

    public function dashboard()
    {
        $this->view('auth/dashboard', get_defined_vars());
    }

    public function broadcast()
    {

        $auth = $this->auth();

        $sieve = $_REQUEST;
        $query = BroadCast::Published()->latest();
        // ->where('status', 1);  //in review
        $page = (isset($_GET['page'])) ? $_GET['page'] : 1;
        $per_page = 50;
        $skip = (($page - 1) * $per_page);

        $data = $query->count();

        $news = $query
            ->offset($skip)
            ->take($per_page)
            ->get();  //filtered


        $this->view('auth/broadcast', compact('news', 'sieve', 'data', 'per_page'));
    }
}
