<?php

namespace v2\Models\Wallet;


use v2\Models\Wallet\Journals;
use v2\Models\Wallet\ChartOfAccount;
use Illuminate\Database\Eloquent\Model as Eloquent;

class JournalInvolvedAccounts extends Eloquent
{

	protected $fillable = [
		'journal_id',
		'chart_of_account_id',
		'description',
		'contact',
		'tax',
		'credit',
		'debit',
		'prior_balance',
		'post_balance',
	];

	protected $table = 'ac_involved_accounts';
	protected $connection = 'wallet';



	public function get_second_leg()
	{

		return self::where('journal_id', $this->journal_id)->where('id', '!=', $this->id)->first();
	}



	public function getformattedPriorBalanceAttribute()
	{
		if ($this->prior_balance < 0) {
			return  "(" . number_format(abs($this->prior_balance), 2) . ")";
		} else {

			return number_format($this->prior_balance, 2);
		}
	}


	public function getformattedPostBalanceAttribute()
	{
		if ($this->post_balance < 0) {
			return  "(" . number_format(abs($this->post_balance), 2) . ")";
		} else {

			return number_format($this->post_balance, 2);
		}
	}



	/**
	 * [$validator_rules this contains validations rules.
	 * it will be used with the Validator classs.
	 * keys are names of form inputs POSTED while values are arrays 
	 * containing rules]
	 * @var [array]
	 */
	public static $validator_rules = [

		'credit' => [
			'numeric' =>  true,
		],

		'debit' => [
			'numeric' =>  true,
		],

		'description' => [
			'required' =>  true,
			'max' =>  500,
			'min' =>  2,
		],

		'chart_of_account_id' => [
			//will be checked in controller manually
		],

	];




	public function delete_involved_account()
	{
		$this->delete();
		//We will do more here like reconcile the current balance of the chart
		//of account. Based on whether 
		//1) this is a credit or a debit post
		//2) chart of account is a credit or debit balance

	}


	public static function create_involved_account($involved_account)
	{
		return  self::create($involved_account);
	}


	public function chart_of_account()
	{
		return $this->belongsTo(ChartOfAccount::class, 'chart_of_account_id');
	}

	public function journal()
	{
		return $this->belongsTo(Journals::class, 'journal_id');
	}
}
