<?php

namespace v2\Models\Wallet;


use User, Config;
use v2\Models\Wallet\JournalInvolvedAccounts;
use Illuminate\Database\Eloquent\Model as Eloquent;

class Journals extends Eloquent
{

	protected $fillable = [
		'user_id',
		'company_id',
		'chart_of_account_id',
		'reference',
		'notes',
		'journal_type',
		'currency_id',
		'published_status',
		'journal_date',
		'attached_files',
	];

	protected $table = 'ac_account_journals';
	protected $connection = 'wallet';



	/**
	 * [$validator_rules this contains validations rules.
	 * it will be used with the Validator classs.
	 * keys are names of form inputs POSTED while values are arrays 
	 * containing rules]
	 * @var [array]
	 */
	public static $validator_rules = [
		'reference' => [
			// 'required' =>  true,
		],

		'notes' => [
			'required' => true,
			'max' =>  500,
		],

		'journal_type' => [
			// 'required' =>  true,
		],

		'journal_date' => [
			'required' =>  true,
		],
		'involved_accounts' => [
			// 'required' => true,
		],

	];



	public function publish()
	{
		$journal = self::find($this->id);
		foreach ($journal->involved_accounts as $involved_account) {

			$chart_of_account =  $involved_account->chart_of_account;
			$chart_of_account->current_balance;

			$involved_account->update(['prior_balance' => $chart_of_account->current_balance]);

			$chart_of_account->post_credit(intval($involved_account->credit));
			$chart_of_account->post_debit(intval($involved_account->debit));

			$involved_account->update(['post_balance' => $chart_of_account->current_balance]);
		}


		$journal->update(['published_status' => 1]);
	}

	public function scopePublished($query)
	{
		return $query->where('published_status', 1);
	}

	public function scopeCompanyJournals($query, $company_id)
	{
		return $query->where('company_id', $company_id);
	}

	public function scopeDateRange($query, $from = null, $to = null)
	{
		if (($from == null) && ($to != null)) {
			return $query->whereDate('journal_date', '<=', $to);
		} elseif (($to == null) && ($from != null)) {
			return $query->whereDate('journal_date', '>=', $from);
		} elseif (($from == null) && ($to == null)) {

			return $query;
		} elseif (($from != null) && ($to != null)) {

			return $query->whereDate('journal_date', '>=', $from)->whereDate('journal_date', '<=', $to);
		}
	}


	public function geteditLinkAttribute()
	{
		return Config::domain() . "/journals/{$this->id}/edit";
	}


	public function getviewLinkAttribute()
	{
		return Config::domain() . "/journals/{$this->id}/view";
	}


	public function getattachmentsAttribute()
	{
		if ($this->attached_files == null) {

			return [];
		}

		return json_decode($this->attached_files, true);
	}


	public function is_published()
	{
		return ($this->published_status == 1);
	}

	public function getpublishedStateAttribute()
	{
		if ($this->is_published()) {
			return "<span class='badge badge-success'>Published</span>";
		} else {
			return  "<span class='badge badge-danger'>Not Published</span>";
		}
	}


	public function getamountAttribute()
	{

		return $this->involved_accounts->sum('credit');
	}



	public function upload_attachments(array $files)
	{
		$directory = 'uploads/journals_attachments';

		foreach ($files as $attribute => $attributes) {
			foreach ($attributes as $key => $value) {
				$refined_file[$key][$attribute] = $value;
			}
		}

		$i = 0;
		foreach ($refined_file as  $file) {

			$handle = new Upload($file);


			$file_type = explode('/', $handle->file_src_mime)[0];
			if (($file_type == 'image') || ($file_type == 'video') || true) {




				$handle->Process($directory);
				$file_path = $directory . '/' . $handle->file_dst_name;

				$attachments[$i] = $file_path;
			}
			$i++;
		}






		$this->update(['attached_files' => json_encode($attachments)]);


		return ($this);
	}


	public function getcreateddateAttribute()
	{
		return date("Y-m-d", strtotime($this->created_at));
	}

	public function involved_accounts()
	{
		return $this->hasMany(JournalInvolvedAccounts::class, 'journal_id');
	}

	public function accountant()
	{
		return $this->belongsTo(User::class, 'user_id');
	}
}
